package drawing;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.File;




public class ColorDialog extends JDialog implements WindowListener {


  private IMainframe mfr;
  private ColorProperties colprops;
  private ColorList currentcl;
  private Container c;


  //Layout-Elemente
  private Choice collistchoice;
  private JLabel lclchoice;
  private ColorListPane clp;
  private JButton bscrollup, bscrolldown, baddcol, bdelcol, bsavecl, bdelcl, bok, babort;


  /*
  *
  *  Konstruktor
  *
  */
  public ColorDialog(IMainframe mfr, ColorList colorlist) {

    super((JFrame)mfr, true);

    this.mfr = mfr;
    this.colprops = new ColorProperties();


    this.currentcl = new ColorList("Aktuelle Einstellungen");
    Color[] colarray = colorlist.getAllColors();
    for (int i = 0; i < colarray.length; i++) {
      currentcl.addColor(colarray[i]);
    }

    createInterface();

  }



  //Erstellt das Layout
  private void createInterface() {

    //Window-Properties
    setResizable(false);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    addWindowListener(this);

    //Containers-Properties
    c = this.getContentPane();
    this.setSize(new Dimension(480, 380));
    c.setLayout(null);
    this.setTitle("Farbeinstellungen");



    //Auswahl der Farbliste
    lclchoice = new JLabel("Farblisten: ");
    lclchoice.setSize(70, 20);
    lclchoice.setLocation(20, 20);

    collistchoice = new Choice();
    collistchoice.setSize(200, 20);
    collistchoice.setLocation(90, 20);

    collistchoice.add("Aktuelle Einstellung");
    String[] items = colprops.getColorListNames();
    for (int i = 0; i < items.length; i++) {
      collistchoice.add(items[i]);
    }
    collistchoice.select("Aktuelle Einstellung");

    collistchoice.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        String curritem = collistchoice.getSelectedItem();
        if (curritem.equals("Aktuelle Einstellung")) {
          clp.setColorList(currentcl);
        }
        else {
          try {
            ColorList cl = colprops.getColorList(curritem);
            clp.setColorList(cl);
          }
          catch (Exception ex) {
            System.out.println(ex.getMessage());
            collistchoice.remove(curritem);
          }
        }
      }
    });




    //ScrollPane in dem die Farben untereinander angezeigt werden.
    clp = new ColorListPane(currentcl, 200, 200);
    clp.setLocation(90, 60);



    //Scroll-Up Button
    try {
      BufferedImage up = ImageIO.read(new File("drawing/up.png"));
      ImageIcon upicon = new ImageIcon(up);
      bscrollup = new JButton(upicon);
    }
    catch (Exception e) {
      bscrollup = new JButton("up");
    }
    bscrollup.setSize(35, 50);
    bscrollup.setLocation(20, 95);
    //bscrollup.setEnabled(false);

    bscrollup.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        clp.scrollUp();
      }
    });


    //Scroll-Down Button
   try {
      BufferedImage down = ImageIO.read(new File("drawing/down.png"));
      ImageIcon downicon = new ImageIcon(down);
      bscrolldown = new JButton(downicon);
    }
    catch (Exception e) {
      bscrolldown = new JButton("down");
    }
    bscrolldown.setSize(35, 50);
    bscrolldown.setLocation(20, 175);

    bscrolldown.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        clp.scrollDown();
      }
    });



    //Farbe-Hinzufgen-Button
    baddcol = new JButton("Farbe hinzufgen");
    baddcol.setSize(150, 30);
    baddcol.setLocation(320, 60);
    baddcol.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        baddcol_actionPerformed();
      }
    });


    bdelcol = new JButton("Farbe lschen");
    bdelcol.setSize(150, 30);
    bdelcol.setLocation(320, 100);
    bdelcol.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        bdelcol_actionPerformed();
      }
    });



    //Aktuelle Farbliste speichern - Button
    bsavecl = new JButton("Farbliste speichern");
    bsavecl.setSize(150, 30);
    bsavecl.setLocation(320, 190);
    bsavecl.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        bsavecl_actionPerformed();
      }
    });


    //Aktuelle Farbliste lschen - Button
    bdelcl = new JButton("Farbliste lschen");
    bdelcl.setSize(150, 30);
    bdelcl.setLocation(320, 230);
    bdelcl.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        bdelcl_actionPerformed();
      }
    });


    //bernehmen-Button
    bok = new JButton("bernehmen");
    bok.setSize(100, 30);
    bok.setLocation(20, 300);
    bok.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        bok_actionPerformed();
      }
    });

    //Abbrechen-Button
    babort = new JButton("Abbrechen");
    babort.setSize(100, 30);
    babort.setLocation(140, 300);
    babort.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        babort_actionPerformed();
      }
    });



    //Elemente dem Container hinzufgen
    c.add(lclchoice);
    c.add(collistchoice);

    c.add(clp);

    c.add(bscrollup);
    c.add(bscrolldown);
    c.add(bdelcol);
    c.add(baddcol);
    c.add(bsavecl);
    c.add(bdelcl);
    c.add(bok);
    c.add(babort);

  }


  //Action des Farbe-Hinzufgen-Buttons
  private void baddcol_actionPerformed() {
    Color newcol = JColorChooser.showDialog(this, "Farbauswahl", Color.BLACK);

    if (newcol != null) {
      ColorList cl = clp.getColorList();
      cl.addColor(newcol);
      clp.setColorList(cl);
    }
  }


  //Action des Farbe-Lschen-Buttons
  private void bdelcol_actionPerformed() {
    clp.deleteCurrentColor();

  }


  //Action des Farbliste-Speichern-Buttons
  private void bsavecl_actionPerformed() {
    boolean incorrect = true;
    String name = null;
    int decision = 1;

    while (incorrect) {
      name = JOptionPane.showInputDialog(this, "Name der Farbliste: ", "Farblistenname",
      JOptionPane.QUESTION_MESSAGE);

      if (name == null) return;

      incorrect = ((name.equals("")) || (name.equals("Aktuelle Einstellung"))
      || (colprops.existsName(name)));

      if (incorrect) {
        decision = JOptionPane.showConfirmDialog(this,
        "Der angegebene Name existiert bereits. Wollen Sie die existierende Farbliste berschreiben?",
        "Falscher Farblistenname", JOptionPane.YES_NO_OPTION);
        System.out.println(decision + "");

        if (decision == 0) { //Ja wurde gewhlt
          break;
        }
      }
    }

    //ColorList newlist = clp.getColorList();
    //newlist.setName(name);

    ColorList newlist = new ColorList(name);

    Color[] colarray = clp.getColorList().getAllColors();

    for (int i = 0; i < colarray.length; i++) {
      newlist.addColor(colarray[i]);
    }

    colprops.addColorList(newlist);

    if (decision != 0) {
      collistchoice.add(name);
    }
    collistchoice.select(name);



  }


  //Action des Farbliste-Lschen-Buttons
  private void bdelcl_actionPerformed() {
    String clname = collistchoice.getSelectedItem();

    if (!(clname.equals("Aktuelle Einstellung"))) {
      try {
        colprops.deleteColorList(clname);
        collistchoice.remove(clname);
        collistchoice.select("Aktuelle Einstellung");
        clp.setColorList(currentcl);
      }
      catch (Exception e) {
        //e.printStackTrace();
      }
    }

  }



  //Action des bernehmen-Buttons
  private void bok_actionPerformed() {
    mfr.setColorList(clp.getColorList());
    dispose();
  }


  //Action des Abbrechen-Buttons
  private void babort_actionPerformed() {
    dispose();
  }



  //WindowListener
  public void windowClosing(WindowEvent event1) {
    dispose();
  }
  public void windowGainedFocus(WindowEvent event2) {}
  public void windowClosed(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}



}