package drawing;


import java.awt.*;
import java.awt.geom.*;

public class SplineThemeRiverDrawer extends ThemeRiverDrawer {



  public SplineThemeRiverDrawer(Point[][] controlpoints, int width, int height, ColorList cl) {
    super(controlpoints, width, height, cl);
  }



  public void draw(Graphics2D g2d) {

    if (controlpoints != null) {

      //GeneralPaths, in welche die Splinezge geschrieben werden
      GeneralPath[] regions = new GeneralPath[controlpoints[0].length];

      //Startbedingungen

      double maxposx = controlpoints[0][0].x;
      CubicCurve2D[] oldcurves = new CubicCurve2D[controlpoints[0].length];

      for (int i = 0; i < controlpoints[0].length; i++) {
        double ypos = controlpoints[0][i].y;

        //Setzen eines Startsplines
        oldcurves[i] = new CubicCurve2D.Double();
        oldcurves[i].setCurve(0,ypos, maxposx/4, ypos, maxposx/2, ypos, maxposx*0.75, ypos);

        //Initialisierung der GeneralPaths
        regions[i] = new GeneralPath();
      }


      //Erstellen der Spline-Pfade
      for (int i = 0; i < controlpoints.length-1; i++) {
        for (int j = 0; j < controlpoints[0].length; j++) {


          Point p1 = controlpoints[i][j];
          Point p4 = controlpoints[i+1][j];

          //1. Kontrollpunkt liegt auf Verlngerung des Startpunkts und des 2. Kontrollpunkts
          //des vorigen Splines
          Point2D.Double p2 = new Point2D.Double(p1.x + (p1.x - oldcurves[j].getCtrlX2()),
          p1.y + (p1.y - oldcurves[j].getCtrlY2()));

          //2. Kontrollpunkt ist das Mittel von p1 und p4
          Point2D.Double p3 = new Point2D.Double(0.5*p1.x + 0.5*p4.x, 0.5*p1.y + 0.5*p4.y);



          CubicCurve2D.Double currcurve = new CubicCurve2D.Double();
          currcurve.setCurve(p1, p2, p3, p4);

          //Neues Setzen des Vorgnger-Splines
          oldcurves[j] = currcurve;
          regions[j].append(currcurve, true);

        }
      }

      //Zeichnen der Flchen
      for (int i = 0; i < controlpoints[0].length-1; i++) {
        //GeneralPath dessen obere Grenze dem oberen Polygonzug entspricht
        GeneralPath currpath = new GeneralPath();
        currpath.append(regions[i], false);

        //Erweitern auf die unteren Ecken des Bildes
        currpath.lineTo((float)width,(float)height);
        currpath.lineTo((float)0,(float)height);
        currpath.closePath();



        //GeneralPath dessen untere Grenze dem unteren Polygonzug entspricht
        GeneralPath currpath2 = new GeneralPath();
        currpath2.append(regions[i+1], false);

        currpath2.lineTo((float)width,(float)0);
        currpath2.lineTo((float)0,(float)0);
        currpath2.closePath();

        //Verunden der beiden GeneralPaths mit Hilfe der Area-Klasse
        Area upper = new Area(currpath);
        upper.intersect(new Area(currpath2));

        g2d.setColor(cl.getColor(i));
        g2d.fill(upper);
      }


      //Seitenrnder gerade schneiden
      g2d.setColor(Color.WHITE);
      //lsche linken Seitenrand
      g2d.fillRect(0, controlpoints[0][0].y, controlpoints[0][0].x-1,
      controlpoints[0][controlpoints[0].length-1].y - controlpoints[0][0].y);

      //lsche rechten Seitenrand
      g2d.fillRect(controlpoints[controlpoints.length-1][0].x+1, controlpoints[controlpoints.length-1][0].y,
      width-(controlpoints[controlpoints.length-1][0].x+1),
      controlpoints[controlpoints.length-1][controlpoints[0].length-1].y - controlpoints[controlpoints.length-1][0].y);


    }

  }



}